@@ -40,6 +40,8 @@ def test_face3d_init():
 
     assert face.area == 4
     assert face.perimeter == 8
+    assert round(face.altitude, 3) == round(math.pi / 2, 3)
+    assert round(face.azimuth, 3) == 0
     assert face.is_clockwise is False
     assert face.is_convex is True
     assert face.is_self_intersecting is False
@@ -591,7 +593,6 @@ def test_corner_vertices():
     face_1 = Face3D(pts_1, plane_1)
     face_2 = Face3D(pts_2, plane_1)
     face_3 = Face3D(pts_1, plane_2)
-    face_4 = Face3D(pts_2, plane_2)
 
     assert face_1.upper_left_corner.is_equivalent(Point3D(2, 0, 2), 1e-6)
     assert face_1.lower_left_corner.is_equivalent(Point3D(2, 0, 0), 1e-6)
@@ -633,12 +634,12 @@ def test_remove_colinear_vertices():
              Point3D(0, 2))
     pts_3 = (Point3D(0, 0), Point3D(2, 0), Point3D(2, 2),
              Point3D(0, 2), Point3D(0, 2))
-    pts_4 = (Point3D (24.21, 23.22, 8.00), Point3D (24.21, 23.22, 11.60),
-             Point3D (24.61, 23.22, 11.60), Point3D (27.85, 23.22, 11.60),
-             Point3D (30.28, 23.22, 11.60), Point3D (30.28, 23.22, 0.00),
-             Point3D (27.85, 23.22, 0.00), Point3D (27.85, 23.22, 4.00),
-             Point3D (27.85, 23.22, 8.00), Point3D (27.85, 23.22, 8.00),
-             Point3D (24.61, 23.22, 8.00), Point3D (24.61, 23.22, 8.00))
+    pts_4 = (Point3D(24.21, 23.22, 8.00), Point3D(24.21, 23.22, 11.60),
+             Point3D(24.61, 23.22, 11.60), Point3D(27.85, 23.22, 11.60),
+             Point3D(30.28, 23.22, 11.60), Point3D(30.28, 23.22, 0.00),
+             Point3D(27.85, 23.22, 0.00), Point3D(27.85, 23.22, 4.00),
+             Point3D(27.85, 23.22, 8.00), Point3D(27.85, 23.22, 8.00),
+             Point3D(24.61, 23.22, 8.00), Point3D(24.61, 23.22, 8.00))
     plane_1 = Plane(Vector3D(0, 0, 1))
     face_1 = Face3D(pts_1, plane_1)
     face_2 = Face3D(pts_2, plane_1)
@@ -1223,28 +1224,40 @@ def test_extract_rectangle():
     face_7 = Face3D(pts_7, plane)
 
     f1_result = face_1.extract_rectangle(0.0001)
-    assert f1_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f1_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f1_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f1_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f1_result[2]) == 0
     f2_result = face_2.extract_rectangle(0.0001)
-    assert f2_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f2_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f2_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f2_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f2_result[2]) == 0
     f3_result = face_3.extract_rectangle(0.0001)
-    assert f3_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f3_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f3_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f3_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f3_result[2]) == 1
     f4_result = face_4.extract_rectangle(0.0001)
-    assert f4_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f4_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f4_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f4_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f4_result[2]) == 2
     f5_result = face_5.extract_rectangle(0.0001)
-    assert f5_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f5_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f5_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f5_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f5_result[2]) == 2
     f6_result = face_6.extract_rectangle(0.0001)
-    assert f6_result[0] == LineSegment3D.from_end_points(Point3D(2, 0, 0), Point3D(0, 0, 0))
-    assert f6_result[1] == LineSegment3D.from_end_points(Point3D(2, 0, 2), Point3D(0, 0, 2))
+    assert f6_result[0] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 0), Point3D(0, 0, 0))
+    assert f6_result[1] == LineSegment3D.from_end_points(
+        Point3D(2, 0, 2), Point3D(0, 0, 2))
     assert len(f6_result[2]) == 1
     f7_result = face_7.extract_rectangle(0.0001)
     assert f7_result is None
@@ -1258,8 +1271,10 @@ def test_extract_rectangle_complex():
     face_1 = Face3D(pts_1, plane)
     f1_result = face_1.extract_rectangle(0.0001)
 
-    assert f1_result[0] == LineSegment3D.from_end_points(Point3D(-1, -1, 0), Point3D(-10, -1, 0))
-    assert f1_result[1] == LineSegment3D.from_end_points(Point3D(-1, -1, 3), Point3D(-10, -1, 3))
+    assert f1_result[0] == LineSegment3D.from_end_points(
+        Point3D(-1, -1, 0), Point3D(-10, -1, 0))
+    assert f1_result[1] == LineSegment3D.from_end_points(
+        Point3D(-1, -1, 3), Point3D(-10, -1, 3))
     assert len(f1_result[2]) == 1
     assert len(f1_result[2][0].vertices) == 4
 
@@ -1271,11 +1286,13 @@ def test_sub_faces_by_ratio_gridded():
 
     sub_faces = face_1.sub_faces_by_ratio_gridded(0.4, 2, 2)
     assert len(sub_faces) == 24
-    assert sum([face.area for face in sub_faces]) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+    assert sum([face.area for face in sub_faces]) == \
+        pytest.approx(face_1.area * 0.4, rel=1e-3)
 
     sub_faces = face_1.sub_faces_by_ratio_gridded(0.4, 12, 12)
     assert len(sub_faces) == 1
-    assert sum([face.area for face in sub_faces]) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+    assert sum([face.area for face in sub_faces]) == \
+        pytest.approx(face_1.area * 0.4, rel=1e-3)
 
 
 def test_sub_faces_by_ratio_rectangle():
@@ -1336,13 +1353,13 @@ def test_sub_faces_by_ratio_sub_rectangle_non_rect():
     assert sum(areas) == pytest.approx(face_1.area * 0.25, rel=1e-3)
     for sf in sub_faces:
         assert face_1.is_sub_face(sf, 0.01, 1)
-    
+
     pts_2 = [
-        Point3D(34.068566268151841,  18.694085210192, 3.9999999999999987), 
-        Point3D(34.068566268151862, 18.694085210192011, 7.0), 
-        Point3D(34.442641391689776, 21.198643189820409, 7.0000000000000018), 
-        Point3D(34.442641391689783, 21.198643189820412, 7.4999999999999973), 
-        Point3D(33.710274088688173, 16.295199316291654, 7.5), 
+        Point3D(34.068566268151841,  18.694085210192, 3.9999999999999987),
+        Point3D(34.068566268151862, 18.694085210192011, 7.0),
+        Point3D(34.442641391689776, 21.198643189820409, 7.0000000000000018),
+        Point3D(34.442641391689783, 21.198643189820412, 7.4999999999999973),
+        Point3D(33.710274088688173, 16.295199316291654, 7.5),
         Point3D(33.710274088688173, 16.295199316291651, 4.0)
     ]
     face_2 = Face3D(pts_2)
@@ -1356,7 +1373,6 @@ def test_sub_faces_by_ratio_sub_rectangle_non_rect():
         assert face_2.is_sub_face(sf, 0.01, 1)
 
 
-
 def test_sub_faces_by_ratio():
     """Test the sub_faces_by_ratio method."""
     pts_1 = (Point3D(0, 0), Point3D(0, 2), Point3D(2, 2), Point3D(2, 0))
@@ -1380,12 +1396,10 @@ def test_sub_faces_by_dimension_rectangle():
     """Test the sub_faces_by_dimension_rectangle method."""
     pts_1 = (Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0))
     pts_2 = (Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(4, 0, 2), Point3D(4, 0, 0))
-    plane = Plane(Vector3D(0, 0, 1))
     face_1 = Face3D(pts_1)
     face_2 = Face3D(pts_2)
     sub_face_height = 1.0
     sill_height = 1.0
-    rect_height = 2.0
     div_dist = 1.0
 
     sub_faces_1 = face_1.sub_faces_by_dimension_rectangle(
@@ -1400,3 +1414,31 @@ def test_sub_faces_by_dimension_rectangle():
     segs_2 = sub_faces_2[0].boundary_segments
     assert segs_2[0].length == 1.0
     assert segs_2[1].length == sub_face_height
+
+
+def test_sub_faces_by_ratio_sub_rectangle_tol_issue():
+    """Test the Face3D sub_faces_by_ratio_sub_rectangle method."""
+    geo_file = './tests/json/face_tolerance_by_ratio.json'
+    with open(geo_file, 'r') as fp:
+        geo_dict = json.load(fp)
+    face_1 = Face3D.from_dict(geo_dict)
+    window_height = 10
+    sill_height = 3
+    horiz_separation = 9
+    vert_separation = 0
+
+    sub_faces = face_1.sub_faces_by_ratio_sub_rectangle(
+        0.4, window_height, sill_height, horiz_separation, vert_separation, 0.0001)
+    assert len(sub_faces) == 9
+    areas = [srf.area for srf in sub_faces]
+    assert sum(areas) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+    for sf in sub_faces:
+        assert face_1.is_sub_face(sf, 0.01, 1)
+
+
+def test_extract_all_from_stl():
+    file_path = 'tests/stl/cube_binary.stl'
+    faces = Face3D.extract_all_from_stl(file_path)
+    assert len(faces) == 12
+    assert all((isinstance(f, Face3D) for f in faces))
+    assert all((len(f) == 3 for f in faces))
